// Copyright (c) 2010  Anna Bigatti, Christof Soeger
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This program shows what can be computed in CoCoALib using Normaliz:\n"
  "a library for computations in affine monoids, vector configurations,\n"
  "lattice polytopes, and rational cones.";

const string LongDescription =
  "NewNormalizConeLong gives more speed \n"
  "NewNormalizConeMPZ  gives reliability \n"
  "";
//----------------------------------------------------------------------

// utility function while waiting for C++Ox vector initializer
vector<ZZ> ZZVec(int* CVector, int len)
{
  vector<ZZ> v;
  for (int i=0; i<len; ++i)  v.push_back(ZZ(CVector[i]));
  return v;
}


void program()
{
  GlobalManager CoCoAFoundations;
  cout << ShortDescription << endl;
  cout << boolalpha; // prints true/false for bool

#ifdef CoCoA_WITH_NORMALIZ
  int M[5][3] = {{2, 0, 0},
                 {1, 3, 1},
                 {3, 3, 1},
                 {0, 0, 2},
                 {1,-3, 1}};

  vector<vector<ZZ> > l;
  for (int i=0; i<5; ++i)  l.push_back(ZZVec(M[i], 3));

  cout << "l -> " << l << endl;

  // waiting for Normaliz Team decisions on input
// 0  Generators, integral closure
// 1  Generators, normalisation
// 2  Polytope
// 3  Rees
// 4  Inequalities
// 5  Equalities
// 6  Congruences
// 10 lattice ideal (toric ring of a binomial ideal)

  double t0;
  int InputType = 1;
  vector<vector<ZZ> > res;
  vector<ZZ>  res2;

  cout << "--- NewNormalizConeLong gives more speed ---" << endl;
  NormalizCone c2(NewNormalizConeLong(l, InputType));
  t0 = CpuTime();
  HilbertBasis(res, c2);
  cout << "time long    -> " << CpuTime()-t0 << endl;
  cout << "HilbertBasis -> " << res << endl;

  cout << "--- NewNormalizConeMPZ  gives reliability ---" << endl;
  NormalizCone C2(NewNormalizConeMPZ(l, InputType));
  t0 = CpuTime();
  HilbertBasis(res, C2);
  cout << "time mpz     -> " << CpuTime()-t0 << endl;
  cout << "HilbertBasis -> " << res << endl;

  cout << endl;
  cout << "More operations on NormalizCone:" << endl;
  Ht1Elements(res, c2);
  cout << "Ht1Elements -> " << res << endl;
  SupportHyperplanes(res, c2);
  cout << "SupportHyperplanes -> " << res << endl;
  HVector(res2, c2);
  cout << "HVector -> " << res2 << endl;
//   Triangulation(res, c2);
//   cout << "Triangulation -> " << res << endl;


#else
  cout << "Normaliz library is not linked." << endl;
#endif
  
	//use it
}





//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-Normaliz1.C,v 1.7 2011/02/17 16:50:04 bigatti Exp $
// $Log: ex-Normaliz1.C,v $
// Revision 1.7  2011/02/17 16:50:04  bigatti
// -- getting ready for new official veson on Normaliz: added HVector, removed Triangulation
//
// Revision 1.6  2010/12/17 16:07:54  abbott
// Ensured that all i/o in examples is on standard C++ streams
// (rather than GlobalInput(), etc).
//
// Revision 1.5  2010/11/22 17:33:31  bigatti
// -- input with c-style array
//
// Revision 1.4  2010/10/12 11:22:54  bigatti
// -- TmpNormaliz.H simplified:
//    now NormalizCone is a smart pointer to NormalizConeBase
//    and the concrete classes are entirely in the .C file
// -- added Ht1Elements, SupportHyperplanes, Triangulation
// -- added some text in ex-Normaliz1 and 2
//
// Revision 1.3  2010/10/08 13:53:51  bigatti
// -- cleaned up
//
// Revision 1.2  2010/10/08 10:39:32  bigatti
// -- extended interface for normaliz
//
// Revision 1.1  2010/10/05 14:31:26  bigatti
// -- first import
//
// Revision 1.4  2008/10/07 12:12:54  abbott
// Removed useless commented out #include.
//
// Revision 1.3  2007/05/31 16:06:16  bigatti
// -- removed previous unwanted checked-in version
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.9  2007/03/07 11:51:40  bigatti
// -- improved test alignment
//
// Revision 1.8  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.7  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.6  2007/03/02 10:47:53  cocoa
// First stage of RingZ modifications -- tests do not compile currently, Anna will fix this.
//
// Revision 1.5  2007/03/01 13:52:59  bigatti
// -- minor: fixed typo
//
// Revision 1.4  2007/02/28 15:15:56  bigatti
// -- minor: removed quotes in description
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
