// Copyright (c) 2010 Anna Bigatti
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "This program shows what can be computed in CoCoALib using Frobby:\n"
  "a library for ...";

const string LongDescription =
  "";
//----------------------------------------------------------------------


void program()
{
  GlobalManager CoCoAFoundations;
  cout << ShortDescription << endl;
  cout << boolalpha; // prints true/false for bool

#ifdef CoCoA_WITH_FROBBY
  PolyRing P = NewPolyRing(RingQ(), 3);
  RingElem x(indet(P,0));
  RingElem y(indet(P,1));
  RingElem z(indet(P,2));
  
  //I := Ideal(x^2, x*y, y^2, z^2);
  ideal I = ideal(x*x, x*y, y*y, z*z);
  cout << "I = " << I << endl << endl;
  
  cout << "AlexanderDualFrobby(I, LPP(x*x*y*y*z*z)) = "
       <<  AlexanderDualFrobby(I, LPP(x*x*y*y*z*z)) << endl << endl;

  cout << "MaximalStandardMonomials(I) = "
       <<  MaximalStandardMonomials(I) << endl << endl;

  vector<ideal> ID;
  IrreducibleDecomposition(ID, I);
  cout << "IrreducibleDecomposition(ID, I):" << endl
       <<  "  ID[0] = " << ID[0] << endl
       <<  "  ID[1] = " << ID[1] << endl << endl;

  vector<ideal> PD;
  PrimaryDecompositionFrobby(PD, I);
  cout << "PrimaryDecompositionFrobby(PD, I):" << endl
       <<  "  PD[0] = " << PD[0] << endl << endl;

  vector<ideal> AP;
  AssociatedPrimes(AP, I);
  cout << "AssociatedPrimes(AP, I):" << endl
       <<  "  AP[0] = " << AP[0] << endl << endl;

  cout << "dimension(ideal(x, y)) = "
       <<  dimension(ideal(x, y)) << endl << endl;

  cout << "MultigradedHilbertPoincareNumerator(ideal(x, y)) = "
       <<  MultigradedHilbertPoincareNumerator(ideal(x, y)) << endl << endl;

  cout << "TotalDegreeHilbertPoincareNumerator(ideal(x,y), x + 2 * y) = "
       <<  TotalDegreeHilbertPoincareNumerator(ideal(x,y), x + 2 * y) << endl << endl;
  
#else
  cout << "Frobby library is not linked." << endl << endl;
#endif
}


//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-frobby1.C,v 1.1 2011/07/20 09:13:51 bigatti Exp $
// $Log: ex-frobby1.C,v $
// Revision 1.1  2011/07/20 09:13:51  bigatti
// -- first import
//
