#ifndef CoCoA_normaliz_H
#define CoCoA_normaliz_H

#ifdef CoCoA_WITH_NORMALIZ

//   Copyright (c)  2010 Anna M. Bigatti, Christof Soeger

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "cone.h" // libnormaliz
#include "CoCoA/SmartPtrIRC.H"

#include <vector>
// using std::vector;

namespace CoCoA
{
  class PPVector;    // forward declaration -- defined in TmpPPVector.H 
  class ZZ;          // forward declaration -- defined in ZZ.H 

  class NormalizConeBase; // fwd decl for SmartPtrIRC
  class NormalizCone
  {
  public:
    explicit NormalizCone(const NormalizConeBase* ptr): mySmartPtr(ptr) {}
    // Default copy ctor works fine.
    // Assignment disabled because SmartPtrIRC has no assignment.
    const NormalizConeBase* operator->() const { return mySmartPtr.operator->(); }  ///< Allow const member fns to be called.
    //    const NormalizConeBase* myRawPtr() const { return mySmartPtr.myRawPtr(); } ///< Used by "downcasting" functions
  private: // data members
    SmartPtrIRC<const NormalizConeBase> mySmartPtr;
  };

  class NormalizConeBase: protected IntrusiveReferenceCount   // abstract class
  {
    friend class SmartPtrIRC<const NormalizConeBase>; // Morally "friend NormalizCone", so it can alter reference count.

  public:
    NormalizConeBase() {}
    virtual ~NormalizConeBase() {}
    //    friend void HilbertBasis(PPVector& v, /*const?*/ NormalizConeBase& c);
    friend void HilbertBasis(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
    friend void Ht1Elements(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
    friend void SupportHyperplanes(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
    friend void HVector(std::vector<ZZ>& v, const NormalizCone& c);
    //    friend void Triangulation(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);

  private:
    virtual void myNormalizComputation(libnormaliz::ConeProperty::Enum CP) const =0;
    virtual void myNormalizComputation(const libnormaliz::ConeProperties& CPs) const =0;
    virtual void myHilbertBasis(std::vector<std::vector<ZZ> >& v) const =0;
    virtual void myHt1Elements(std::vector<std::vector<ZZ> >& v) const =0;
    virtual void mySupportHyperplanes(std::vector<std::vector<ZZ> >& v) const =0;
    virtual void myHVector(std::vector<ZZ>& v) const =0;
    //    virtual void myTriangulation(std::vector<std::vector<ZZ> >& v) const =0;
  };

  NormalizCone NewNormalizConeLong(const std::vector<std::vector<ZZ> >& v, int InputType);
  NormalizCone NewNormalizConeMPZ(const std::vector<std::vector<ZZ> >& v, int InputType);
  void HilbertBasis(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
  void Ht1Elements(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
  void SupportHyperplanes(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
  void HVector(std::vector<ZZ>& v, const NormalizCone& c);
  //  void Triangulation(std::vector<std::vector<ZZ> >& v, const NormalizCone& c);
  
} // end of namespace CoCoA

#endif

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/include/CoCoA/ExternalLibs-Normaliz.H,v 1.2 2011/02/17 16:50:04 bigatti Exp $
// $Log: ExternalLibs-Normaliz.H,v $
// Revision 1.2  2011/02/17 16:50:04  bigatti
// -- getting ready for new official veson on Normaliz: added HVector, removed Triangulation
//
// Revision 1.1  2010/11/05 14:27:18  bigatti
// -- was TmpNormaliz**
//
// Revision 1.5  2010/10/12 11:22:54  bigatti
// -- TmpNormaliz.H simplified:
//    now NormalizCone is a smart pointer to NormalizConeBase
//    and the concrete classes are entirely in the .C file
// -- added Ht1Elements, SupportHyperplanes, Triangulation
// -- added some text in ex-Normaliz1 and 2
//
// Revision 1.4  2010/10/08 16:34:08  bigatti
// -- activated NormalizConeMPZ
//
// Revision 1.3  2010/10/08 10:39:32  bigatti
// -- extended interface for normaliz
//

#endif
