//   Copyright (c)  2009 Anna Bigatti

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.


#include "CoCoA/BuildInfo.H"
#include "CoCoA/GlobalManager.H"
#include "CoCoA/error.H"

#ifdef CoCoA_WITH_NORMALIZ
#include "CoCoA/ExternalLibs-Normaliz.H"
#endif

using namespace CoCoA;

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;
#ifdef CoCoA_WITH_NORMALIZ
//#include "CoCoA/io.H"
#include "CoCoA/utils.H"
#include <vector>
using std::vector;
#endif
//----------------------------------------------------------------------
// First test for Normaliz library.
// Trivial computation to test proper integration.
//----------------------------------------------------------------------

#define TEST_ASSERT(cond)   do { if (!(cond)) CoCoA::AssertionFailed(#cond, __FILE__, __LINE__); } while(0)


#ifdef CoCoA_WITH_NORMALIZ
vector<ZZ> ZZVec(int* CVector, int len)
{
  vector<ZZ> v;
  for (int i=0; i<len; ++i)  v.push_back(ZZ(CVector[i]));
  return v;
}
#endif

void program()
{
  GlobalManager CoCoAFoundations;

#ifdef CoCoA_WITH_NORMALIZ
  int M[5][3] = {{2, 0, 0},
                 {1, 3, 1},
                 {3, 3, 1},
                 {0, 0, 2},
                 {1,-3, 1}};

  vector<vector<ZZ> > l;
  for (int i=0; i<5; ++i)  l.push_back(ZZVec(M[i], 3));

  //  cout << "l -> " << l << endl;

  // waiting for Normaliz Team decisions on input
// 0  Generators, integral closure
// 1  Generators, normalisation
// 2  Polytope
// 3  Rees
// 4  Inequalities
// 5  Equalities
// 6  Congruences
// 10 lattice ideal (toric ring of a binomial ideal)

  int InputType = 1;
  vector<vector<ZZ> > reslong;
  vector<vector<ZZ> > resmpz;
  vector<ZZ>  res2;

  NormalizCone c2(NewNormalizConeLong(l, InputType));
  HilbertBasis(reslong, c2);

  NormalizCone C2(NewNormalizConeMPZ(l, InputType));
  HilbertBasis(resmpz, C2);
  TEST_ASSERT(len(resmpz) == len(reslong));
  for (long i=0; i<len(reslong); ++i)
  {
    TEST_ASSERT(len(resmpz[i]) == len(reslong[i]));
    for (long j=0; j<len(reslong[i]); ++j)
    TEST_ASSERT(resmpz[i][j] == reslong[i][j]);
  }
  Ht1Elements(reslong, c2);
  Ht1Elements(resmpz, C2);
  TEST_ASSERT(len(resmpz) == len(reslong));
  for (long i=0; i<len(reslong); ++i)
  {
    TEST_ASSERT(len(resmpz[i]) == len(reslong[i]));
    for (long j=0; j<len(reslong[i]); ++j)
    TEST_ASSERT(resmpz[i][j] == reslong[i][j]);
  }
  SupportHyperplanes(reslong, c2);
  SupportHyperplanes(resmpz, C2);
  TEST_ASSERT(len(resmpz) == len(reslong));
  for (long i=0; i<len(reslong); ++i)
  {
    TEST_ASSERT(len(resmpz[i]) == len(reslong[i]));
    for (long j=0; j<len(reslong[i]); ++j)
    TEST_ASSERT(resmpz[i][j] == reslong[i][j]);
  }
  HVector(res2, c2);

#endif
}


//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/tests/test-normaliz1.C,v 1.2 2011/07/20 10:12:17 bigatti Exp $
// $Log: test-normaliz1.C,v $
// Revision 1.2  2011/07/20 10:12:17  bigatti
// -- fixed compilation without normaliz
//
// Revision 1.1  2011/07/19 16:24:16  bigatti
// -- first import
//
// Revision 1.8  2011/07/05 15:15:33  bigatti
// -- changed AlexanderDual --> AlexanderDualFrobby
// -- changed PrimaryDecomposition --> PrimaryDecompositionFrobby
//
// Revision 1.7  2010/12/17 16:06:25  abbott
// Ensured that all i/o is on standard C++ streams (instead of GlobalInput, etc)
//
// Revision 1.6  2010/11/26 15:32:16  bigatti
// -- renamed Dimension into dimension (coding conventions)
//
// Revision 1.5  2010/11/22 17:39:15  bigatti
// -- updated "TmpFrobby" --> "ExternalLibs-Frobby"
//
// Revision 1.4  2010/02/04 09:38:28  bigatti
// -- new syntax for frobby (more CoCoALib-like)
//
// Revision 1.3  2010/02/03 18:00:22  bigatti
// -- more functions from frobby
//
// Revision 1.2  2009/07/30 15:41:25  bigatti
// -- now using the new nice constructors for ideals
//
// Revision 1.1  2009/02/11 15:08:26  bigatti
// -- first import
//

